%% Raw image
R = imread('_DSC0005.pgm');

%% Gamma correction
R = imadjust(R, [],[],1/2.2);

%% Create color masks
mask = false(2*ceil(size(R,1)/2), 2*ceil(size(R,2)/2), 3);
mask(:,:,1) = repmat(logical([0 0; 0 1]), ceil(size(R,1)/2), ceil(size(R,2)/2));
mask(:,:,2) = repmat(logical([0 1; 1 0]), ceil(size(R,1)/2), ceil(size(R,2)/2));
mask(:,:,3) = repmat(logical([1 0; 0 0]), ceil(size(R,1)/2), ceil(size(R,2)/2));
mask = mask(1:size(R,1), 1:size(R,2), :);

%% Create color channels

I = cat(3,R,R,R);

for i=1:3
    I(:,:,i) = I(:,:,i) .* uint16(mask(:,:,i));    
end

%% Interpolation
%I(2:end-1,:,1) = I(2:end-1,:,1) + I(1:end-2,:,1)/2 + I(3:end,:,1)/2;
I(2:end-1,:,[1 3]) = imlincomb(...
    1,   I(2:end-1,:,[1 3]),...
    0.5, I(1:end-2,:,[1 3]),...
    0.5, I(3:end,:,[1 3]));

I(:,2:end-1,[1 3]) = imlincomb(...
    1,   I(:,2:end-1,[1 3]),...
    0.5, I(:,1:end-2,[1 3]),...
    0.5, I(:,3:end,[1 3]));

I(2:end-1,2:end-1,2) = imlincomb(...
    1,    I(2:end-1,2:end-1,2),...
    0.25, I(1:end-2,2:end-1,2),...
    0.25, I(3:end,2:end-1,2),...
    0.25, I(2:end-1,1:end-2,2),...
    0.25, I(2:end-1,3:end,2));

%% Median filtering

Trgb2yuv = [ 0.299 0.587 0.114; 
    -0.14713 -0.28886 0.436;
    0.615 -0.51499 -0.10001];

I = im2single(I);
tmp = I;

for p=1:3
    tmp(:,:,p) = imlincomb(...
        Trgb2yuv(p,1),I(:,:,1),...
        Trgb2yuv(p,2),I(:,:,2),...
        Trgb2yuv(p,3),I(:,:,3));    
end

tmp(:,:,2) = medfilt2(tmp(:,:,2),[5 5]);
tmp(:,:,3) = medfilt2(tmp(:,:,3),[5 5]);

%% Reconstruct luminance

R = im2single(R);

Tyuv2rgb = inv(Trgb2yuv);

Y = tmp(:,:,1);
U = tmp(:,:,2);
V = tmp(:,:,3);

for p=1:3
    Y(mask(:,:,p)) = (R(mask(:,:,p)) - ...
        Tyuv2rgb(p,2)*U(mask(:,:,p)) - ...
        Tyuv2rgb(p,3)*V(mask(:,:,p))) / Tyuv2rgb(p,1);
end

I = cat(3,Y,U,V);

for p=1:3
    tmp(:,:,p) = imlincomb(...
        Tyuv2rgb(p,1),I(:,:,1),...
        Tyuv2rgb(p,2),I(:,:,2),...
        Tyuv2rgb(p,3),I(:,:,3));    
end









